# Copyright 2021 Damien Nguyen
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import astroid
import pylint.testutils
import pytest

import pylint_secure_coding_standard as pylint_scs

_os_function_strings = {
    'mkdir': (
        'os.mkdir("/tmp/test", 0o777)',
        'os.mkdir(dir_name, 0o777)',
        'os.mkdir("/tmp/test", mode=0o777)',
        'os.mkdir(dir_name, mode=0o777)',
        'os.makedirs("/tmp/test/utils", 0o777)',
        'os.makedirs(dir_name, 0o777)',
        'os.makedirs("/tmp/test/utils", mode=0o777)',
        'os.makedirs(dir_name, mode=0o777)',
    ),
    'mkfifo': (
        'os.mkfifo("/tmp/test.log", 0o777)',
        'os.mkfifo(dir_name, 0o777)',
        'os.mkfifo("/tmp/test.log", mode=0o777)',
        'os.mkfifo(dir_name, mode=0o777)',
    ),
    'mknod': (
        'os.mknod("/tmp/test", 0o777)',
        'os.mknod(dir_name, 0o777)',
        'os.mknod("/tmp/test", mode=0o777)',
        'os.mknod(dir_name, mode=0o777)',
    ),
}


try:
    from pylint.testutils import MessageTest
except ImportError:
    from pylint.testutils import Message as MessageTest


class TestSecureCodingStandardChecker(pylint.testutils.CheckerTestCase):
    CHECKER_CLASS = pylint_scs.SecureCodingStandardChecker

    @pytest.mark.parametrize('platform', ['Linux', 'Darwin', 'Java', 'Windows'])
    @pytest.mark.parametrize('function', ['mkdir', 'mkfifo', 'mknod'])
    @pytest.mark.parametrize('option', [False, True])
    @pytest.mark.parametrize(
        's',
        [
            # mkdir
            'os.mkdir("/tmp/test")',
            'os.mkdir(dir_name)',
            'os.mkdir("/tmp/test", 0o644)',
            'os.mkdir(dir_name, 0o644)',
            'os.mkdir("/tmp/test", mode=mode)',
            'os.mkdir(dir_name, mode=mode)',
            # makedirs
            'os.makedirs("/tmp/test/utils")',
            'os.makedirs(dir_name)',
            'os.makedirs("/tmp/test/utils", 0o644)',
            'os.makedirs(dir_name, 0o644)',
            'os.makedirs("/tmp/test/utils", mode=mode)',
            'os.makedirs(dir_name, mode=mode)',
            # mkfifo
            'os.mkfifo("/tmp/test/utils")',
            'os.mkfifo("/tmp/test/utils", 0o644)',
            'os.mkfifo("/tmp/test/utils", mode=mode)',
            # mknod
            'os.mknod(dir_name)',
            'os.mknod(dir_name, 0o644)',
            'os.mknod(dir_name, mode=mode)',
        ],
    )
    def test_os_function_ok(self, mocker, platform, function, option, s):
        mocker.patch('platform.system', return_value=platform)
        getattr(self.checker, f'set_os_{function}_allowed_modes')(str(option))

        node = astroid.extract_node(s + ' #@')

        with self.assertNoMessages():
            self.checker.visit_call(node)

    @pytest.mark.parametrize(
        ('platform', 'enabled_platform'),
        [
            ('Linux', True),
            ('Darwin', True),
            ('Java', False),
            ('Windows', False),
        ],
    )
    @pytest.mark.parametrize(
        'option',
        [False, True],
    )
    @pytest.mark.parametrize(
        ('function', 's'), [(function, s) for function, tests in _os_function_strings.items() for s in tests]
    )
    def test_os_function_call(self, mocker, platform, enabled_platform, function, option, s):
        mocker.patch('platform.system', return_value=platform)
        getattr(self.checker, f'set_os_{function}_allowed_modes')(str(option))

        print(s + ' #@')
        node = astroid.extract_node(s + ' #@')
        if enabled_platform and option:
            with self.assertAddsMessages(
                MessageTest(
                    msg_id=f'os-{function}-unsafe-permissions',
                    node=node,
                    args=(getattr(self.checker, f'_os_{function}_msg_arg'),),
                ),
                ignore_position=True,
            ):
                self.checker.visit_call(node)
        else:
            with self.assertNoMessages():
                self.checker.visit_call(node)



# Copyright 2021 Damien Nguyen
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import astroid
import pylint.testutils
import pytest

import pylint_secure_coding_standard as pylint_scs

try:
    from pylint.testutils import MessageTest
except ImportError:
    from pylint.testutils import Message as MessageTest


class TestSecureCodingStandardChecker(pylint.testutils.CheckerTestCase):
    CHECKER_CLASS = pylint_scs.SecureCodingStandardChecker

    def test_pickle_load_ok(self):
        nodes = astroid.extract_node(
            '''
            int(0) #@
            foo() #@
            pickle.dump(data, "file.txt") #@
            pickle.dumps(data) #@
            '''
        )

        with self.assertNoMessages():
            for node in nodes:
                self.checker.visit_call(node)

    @pytest.mark.parametrize(
        's',
        [
            'pickle.load("file.txt")',
            r'pickle.loads(b"\x80\x04K\x01.")',
            'pickle.loads(data)',
        ],
    )
    def test_pickle_load_not_ok(self, s):
        node = astroid.extract_node(s + ' #@')
        with self.assertAddsMessages(MessageTest(msg_id='avoid-pickle-load', node=node), ignore_position=True):
            self.checker.visit_call(node)

    @pytest.mark.parametrize(
        's',
        [
            'from pickle import load',
            'from pickle import loads',
            'from pickle import dump, load',
        ],
    )
    def test_pickle_open_importfrom(self, s):
        node = astroid.extract_node(s + ' #@')
        with self.assertAddsMessages(MessageTest(msg_id='avoid-pickle-load', node=node), ignore_position=True):
            self.checker.visit_importfrom(node)



# Copyright 2021 Damien Nguyen
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import astroid
import pylint.testutils
import pytest

import pylint_secure_coding_standard as pylint_scs

try:
    from pylint.testutils import MessageTest
except ImportError:
    from pylint.testutils import Message as MessageTest


class TestSecureCodingStandardChecker(pylint.testutils.CheckerTestCase):
    CHECKER_CLASS = pylint_scs.SecureCodingStandardChecker

    @pytest.mark.parametrize(
        'arg',
        ['True', '0o644', '0o644,'],
    )
    def test_os_open_ok(self, arg):
        nodes = astroid.extract_node(
            '''
            int(0) #@
            foo() #@
            os.open("file.txt") #@
            os.open("file.txt", flags, mode) #@
            os.open("file.txt", os.O_RDONLY) #@
            os.open("file.txt", os.O_RDONLY, mode) #@
            os.open("file.txt", os.O_RDONLY, 0o644) #@
            os.open("file.txt", os.O_RDONLY | os.O_NOFOLLOW) #@
            os.open("file.txt", os.O_RDONLY | os.O_NOFOLLOW, mode) #@
            os.open("file.txt", os.O_RDONLY | os.O_NOFOLLOW, 0o644) #@
            os.open("file.txt", os.O_RDONLY, mode=mode) #@
            os.open("file.txt", os.O_RDONLY, mode=0o644) #@
            os.open("file.txt", os.O_RDONLY | os.O_NOFOLLOW, mode=mode) #@
            os.open("file.txt", os.O_RDONLY | os.O_NOFOLLOW, mode=0o644) #@
            bla.open("file.txt") #@
            bla.open("file.txt", os.O_RDONLY) #@
            bla.open("file.txt", flags=os.O_RDONLY) #@
            bla.open("file.txt", os.O_RDONLY, mode) #@
            bla.open("file.txt", os.O_RDONLY, 0o644) #@
            bla.open("file.txt", os.O_RDONLY, 0o777) #@
            bla.open("file.txt", os.O_RDONLY | os.O_NOFOLLOW) #@
            bla.open("file.txt", os.O_RDONLY | os.O_NOFOLLOW, mode) #@
            bla.open("file.txt", os.O_RDONLY | os.O_NOFOLLOW, 0o644) #@
            bla.open("file.txt", os.O_RDONLY | os.O_NOFOLLOW, 0o777) #@
            bla.open("file.txt", os.O_RDONLY, mode=mode) #@
            bla.open("file.txt", os.O_RDONLY, mode=0o644) #@
            bla.open("file.txt", os.O_RDONLY, mode=0o777) #@
            bla.open("file.txt", os.O_RDONLY | os.O_NOFOLLOW) #@
            bla.open("file.txt", os.O_RDONLY | os.O_NOFOLLOW, mode=mode) #@
            bla.open("file.txt", os.O_RDONLY | os.O_NOFOLLOW, mode=0o644) #@
            bla.open("file.txt", os.O_RDONLY | os.O_NOFOLLOW, mode=0o777) #@
            with os.open("file.txt") as fd: fd.read() #@
            with os.open("file.txt", flags, mode) as fd: fd.read() #@
            with os.open("file.txt", os.O_RDONLY) as fd: fd.read() #@
            with os.open("file.txt", os.O_RDONLY, mode) as fd: fd.read() #@
            with os.open("file.txt", os.O_RDONLY, 0o644) as fd: fd.read() #@
            with os.open("file.txt", os.O_RDONLY | os.O_NOFOLLOW) as fd: fd.read() #@
            with os.open("file.txt", os.O_RDONLY | os.O_NOFOLLOW, mode) as fd: fd.read() #@
            with os.open("file.txt", os.O_RDONLY | os.O_NOFOLLOW, 0o644) as fd: fd.read() #@
            with os.open("file.txt", flags=flags, mode=mode) as fd: fd.read() #@
            with os.open("file.txt", flags=os.O_RDONLY, mode=mode) as fd: fd.read() #@
            with os.open("file.txt", flags=os.O_RDONLY, mode=0o644) as fd: fd.read() #@
            with os.open("file.txt", flags=os.O_RDONLY | os.O_NOFOLLOW, mode=mode) as fd: fd.read() #@
            with os.open("file.txt", flags=os.O_RDONLY | os.O_NOFOLLOW, mode=0o644) as fd: fd.read() #@
            with bla.open("file.txt") as fd: fd.read() #@
            with bla.open("file.txt", os.O_RDONLY) as fd: fd.read() #@
            with bla.open("file.txt", os.O_RDONLY, mode) as fd: fd.read() #@
            with bla.open("file.txt", os.O_RDONLY, 0o644) as fd: fd.read() #@
            with bla.open("file.txt", os.O_RDONLY, 0o777) as fd: fd.read() #@
            with bla.open("file.txt", os.O_RDONLY | os.O_NOFOLLOW) as fd: fd.read() #@
            with bla.open("file.txt", os.O_RDONLY | os.O_NOFOLLOW, mode) as fd: fd.read() #@
            with bla.open("file.txt", os.O_RDONLY | os.O_NOFOLLOW, 0o644) as fd: fd.read() #@
            with bla.open("file.txt", os.O_RDONLY | os.O_NOFOLLOW, 0o777) as fd: fd.read() #@
            with bla.open("file.txt", flags=os.O_RDONLY) as fd: fd.read() #@
            with bla.open("file.txt", flags=os.O_RDONLY, mode=mode) as fd: fd.read() #@
            with bla.open("file.txt", flags=os.O_RDONLY, mode=0o644) as fd: fd.read() #@
            with bla.open("file.txt", flags=os.O_RDONLY, mode=0o777) as fd: fd.read() #@
            with bla.open("file.txt", flags=os.O_RDONLY | os.O_NOFOLLOW, mode=mode) as fd: fd.read() #@
            with bla.open("file.txt", flags=os.O_RDONLY | os.O_NOFOLLOW, mode=0o644) as fd: fd.read() #@
            with bla.open("file.txt", flags=os.O_RDONLY | os.O_NOFOLLOW, mode=0o777) as fd: fd.read() #@
            '''
        )

        self.checker.set_os_open_allowed_modes(arg)

        call_nodes = []
        with_nodes = []

        # Find index of first line starting with 'with'
        for node in nodes:
            if isinstance(node, astroid.With):
                with_nodes.append(node)
            else:
                call_nodes.append(node)

        with self.assertNoMessages():
            for _idx, node in enumerate(call_nodes):
                self.checker.visit_call(node)
            for _idx, node in enumerate(with_nodes):
                self.checker.visit_with(node)

    # ==========================================================================

    @pytest.mark.parametrize('mode', range(0o756, 0o777 + 1), ids=lambda arg: oct(arg))
    @pytest.mark.parametrize(
        ('arg', 'expected_warning'),
        [
            ('False', False),
            ('True', True),
        ],
    )
    def test_os_open_call_default_modes(self, mode, arg, expected_warning):
        code = f'os.open("file.txt", os.O_WRONLY, 0o{mode:o}) #@'
        print(code)
        node = astroid.extract_node(code)
        self.checker.set_os_open_allowed_modes(arg)
        if expected_warning:
            with self.assertAddsMessages(
                MessageTest(msg_id='os-open-unsafe-permissions', node=node, args=(self.checker._os_open_msg_arg,)),
                ignore_position=True,
            ):
                self.checker.visit_call(node)
        else:
            with self.assertNoMessages():
                self.checker.visit_call(node)

    # --------------------------------------------------------------------------

    @pytest.mark.parametrize('mode', range(0o750, 0o761), ids=lambda arg: oct(arg))
    @pytest.mark.parametrize(
        'call_mode',
        [
            'os.open("file.txt", os.O_WRONLY, 0o{:o}) #@',
            'os.open("file.txt", flags=os.O_WRONLY, mode=0o{:o}) #@',
        ],
        ids=('args', 'keyword'),
    )
    @pytest.mark.parametrize(
        ('arg', 'expected_warning'),
        [
            ('False', False),
            ('0o755,', True),
        ],
        ids=('False-False', '[0o755]-True'),
    )
    def test_os_open_call(self, mode, call_mode, arg, expected_warning):
        node = astroid.extract_node(call_mode.format(mode))
        self.checker.set_os_open_allowed_modes(arg)
        if expected_warning and mode != 0o755:
            with self.assertAddsMessages(
                MessageTest(msg_id='os-open-unsafe-permissions', node=node, args=(self.checker._os_open_msg_arg,)),
                ignore_position=True,
            ):
                self.checker.visit_call(node)
        else:
            with self.assertNoMessages():
                self.checker.visit_call(node)

    # ==========================================================================

    @pytest.mark.parametrize('mode', range(0o756, 0o777 + 1), ids=lambda arg: oct(arg))
    @pytest.mark.parametrize(
        'call_mode',
        [
            'with os.open("file.txt", os.O_WRONLY, 0o{:o}) as fd: fd.read() #@',
            'with os.open("file.txt", flags=os.O_WRONLY, mode=0o{:o}) as fd: fd.read() #@',
        ],
        ids=('args', 'keyword'),
    )
    @pytest.mark.parametrize(
        ('arg', 'expected_warning'),
        [
            ('False', False),
            ('True', True),
        ],
    )
    def test_os_open_with_default_modes(self, mode, call_mode, arg, expected_warning):
        node = astroid.extract_node(call_mode.format(mode))
        self.checker.set_os_open_allowed_modes(arg)
        if expected_warning:
            with self.assertAddsMessages(
                MessageTest(msg_id='os-open-unsafe-permissions', node=node), ignore_position=True
            ):
                self.checker.visit_with(node)
        else:
            with self.assertNoMessages():
                self.checker.visit_with(node)

    # --------------------------------------------------------------------------

    @pytest.mark.parametrize('mode', range(0o750, 0o761), ids=lambda arg: oct(arg))
    @pytest.mark.parametrize(
        ('arg', 'expected_warning'),
        [
            ('False', False),
            ('0o755,', True),
        ],
    )
    def test_os_open_with(self, mode, arg, expected_warning):
        node = astroid.extract_node(f'with os.open("file.txt", os.O_WRONLY, 0o{mode:o}) as fd: fd.read() #@')
        self.checker.set_os_open_allowed_modes(arg)
        if expected_warning and mode != 0o755:
            with self.assertAddsMessages(
                MessageTest(msg_id='os-open-unsafe-permissions', node=node), ignore_position=True
            ):
                self.checker.visit_with(node)
        else:
            with self.assertNoMessages():
                self.checker.visit_with(node)



# Copyright 2021 Damien Nguyen
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import pylint.testutils
import pytest

import pylint_secure_coding_standard as pylint_scs

_default_modes = list(range(pylint_scs.SecureCodingStandardChecker.DEFAULT_MAX_MODE + 1))


def _id_func(arg):
    _max_len = 4
    if arg == _default_modes:
        return 'default_modes'
    if isinstance(arg, list) and len(arg) > _max_len:
        return '[{}...{}]'.format(
            ','.join(str(val) for val in arg[:_max_len]),
            ','.join(str(val) for val in arg[-_max_len:]),
        )
    return str(arg)


class TestSecureCodingStandardChecker(pylint.testutils.CheckerTestCase):
    CHECKER_CLASS = pylint_scs.SecureCodingStandardChecker

    @pytest.mark.parametrize(
        ('arg', 'expected'),
        [
            ('0', 0),
            ('false', None),
            ('False', None),
            ('n', None),
            ('no', None),
            ('No', None),
            ('NO', None),
            ('y', _default_modes),
            ('yes', _default_modes),
            ('Yes', _default_modes),
            ('YES', _default_modes),
            ('true', _default_modes),
            ('True', _default_modes),
            ('1', 1),
            ('493', 493),
            ('0o755', 0o755),
            ('0o755,', [0o755]),
            ('0o644, 0o755,', [0o644, 0o755]),
        ],
        ids=_id_func,
    )
    def test_read_octal_mode_option(self, arg, expected):
        print(f'INFO: expected: {expected}')
        assert pylint_scs._read_octal_mode_option('test', arg, _default_modes) == expected

    @pytest.mark.parametrize('arg', ['', ',', ',,', 'nope', 'asd', 'a,', '493, a'])
    def test_read_octal_mode_option_invalid(self, arg):
        # with pytest.raises(ValueError, match='^Unable to convert .* elements to integers!$'):
        with pytest.raises(ValueError, match='^(Invalid value for|Calculated empty value for|Unable to convert).*'):
            pylint_scs._read_octal_mode_option('test', arg, _default_modes)

    @pytest.mark.parametrize('function', ['open', 'mkdir', 'mkfifo', 'mknod'])
    @pytest.mark.parametrize(
        ('arg', 'allowed_modes'),
        [
            ('n', []),
            ('y', _default_modes),
            ('0', []),
            ('0o755', _default_modes),
            ('0o644, 0o755,', [0o644, 0o755]),
        ],
        ids=_id_func,
    )
    def test_os_allowed_mode(self, function, arg, allowed_modes):
        print(f'INFO: allowed_modes: {allowed_modes}')
        getattr(self.checker, f'set_os_{function}_allowed_modes')(arg)
        assert getattr(self.checker, f'_os_{function}_modes_allowed') == allowed_modes



# Copyright 2021 Damien Nguyen
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import astroid
import pylint.testutils
import pytest

import pylint_secure_coding_standard as pylint_scs

try:
    from pylint.testutils import MessageTest
except ImportError:
    from pylint.testutils import Message as MessageTest


class TestSecureCodingStandardChecker(pylint.testutils.CheckerTestCase):
    CHECKER_CLASS = pylint_scs.SecureCodingStandardChecker

    def test_shell_true_ok(self):
        nodes = astroid.extract_node(
            '''
            subprocess.Popen(["cat", "/etc/passwd"], b, e, i, o, e, pre, c) #@
            subprocess.Popen(["cat", "/etc/passwd"], b, e, i, o, e, pre, c, False) #@
            subprocess.Popen(["cat", "/etc/passwd"], b, e, i, o, e, pre, c, False, cwd) #@
            subprocess.Popen(["cat", "/etc/passwd"], shell=False) #@
            sp.Popen(["cat", "/etc/passwd"], shell=False) #@
            subprocess.run(["cat", "/etc/passwd"], shell=False) #@
            sp.run(["cat", "/etc/passwd"], shell=False) #@
            subprocess.call(["cat", "/etc/passwd"], shell=False) #@
            sp.call(["cat", "/etc/passwd"], shell=False) #@
            subprocess.check_call(["cat", "/etc/passwd"], shell=False) #@
            sp.check_call(["cat", "/etc/passwd"], shell=False) #@
            subprocess.check_output(["cat", "/etc/passwd"], shell=False) #@
            sp.check_output(["cat", "/etc/passwd"], shell=False) #@
            '''
        )

        with self.assertNoMessages():
            for node in nodes:
                self.checker.visit_call(node)

    @pytest.mark.parametrize(
        ('s', 'msg_id'),
        [
            ('from os import system', 'avoid-os-system'),
            ('from os import system as os_system', 'avoid-os-system'),
            ('from os import popen', 'avoid-os-popen'),
            ('from os import popen as os_popen', 'avoid-os-popen'),
            ('from subprocess import getoutput', 'avoid-shell-true'),
            ('from subprocess import getoutput as sp_getoutput', 'avoid-shell-true'),
            ('from subprocess import getstatusoutput', 'avoid-shell-true'),
            ('from subprocess import getstatusoutput as sp_getstatusoutput', 'avoid-shell-true'),
            ('from asyncio import create_subprocess_shell', 'avoid-shell-true'),
            ('from asyncio import create_subprocess_shell as create_sp_shell', 'avoid-shell-true'),
        ],
    )
    def test_shell_true_importfrom(self, s, msg_id):
        node = astroid.extract_node(s + ' #@')
        with self.assertAddsMessages(MessageTest(msg_id=msg_id, node=node), ignore_position=True):
            self.checker.visit_importfrom(node)

    @pytest.mark.parametrize(
        ('s', 'msg_id'),
        [
            ('os.system("ls -l")', 'avoid-os-system'),
            ('subprocess.Popen(["cat", "/etc/passwd"], b, e, i, o, e, pre, c, True)', 'avoid-shell-true'),
            ('subprocess.Popen(["cat", "/etc/passwd"], b, e, i, o, e, pre, c, True, cwd)', 'avoid-shell-true'),
            ('subprocess.run(["cat", "/etc/passwd"], shell=True)', 'avoid-shell-true'),
            ('sp.run(["cat", "/etc/passwd"], shell=True)', 'avoid-shell-true'),
            ('subprocess.call(["cat", "/etc/passwd"], shell=True)', 'avoid-shell-true'),
            ('sp.call(["cat", "/etc/passwd"], shell=True)', 'avoid-shell-true'),
            ('subprocess.check_call(["cat", "/etc/passwd"], shell=True)', 'avoid-shell-true'),
            ('sp.check_call(["cat", "/etc/passwd"], shell=True)', 'avoid-shell-true'),
            ('subprocess.check_output(["cat", "/etc/passwd"], shell=True)', 'avoid-shell-true'),
            ('sp.check_output(["cat", "/etc/passwd"], shell=True)', 'avoid-shell-true'),
            ('subprocess.getoutput("ls /bin/ls")', 'avoid-shell-true'),
            ('sp.getoutput("ls /bin/ls")', 'avoid-shell-true'),
            ('subprocess.getstatusoutput("ls /bin/ls")', 'avoid-shell-true'),
            ('sp.getstatusoutput("ls /bin/ls")', 'avoid-shell-true'),
            ('asyncio.create_subprocess_shell("ls /bin/ls")', 'avoid-shell-true'),
            ('asyncio.create_subprocess_shell(cmd)', 'avoid-shell-true'),
            ('asyncio.create_subprocess_shell("ls /bin/ls", stdin=PIPE, stdout=PIPE)', 'avoid-shell-true'),
            ('asyncio.create_subprocess_shell(cmd, stdin=PIPE, stdout=PIPE)', 'avoid-shell-true'),
            ('loop.subprocess_shell(asyncio.SubprocessProtocol, "ls /bin/ls")', 'avoid-shell-true'),
            ('loop.subprocess_shell(asyncio.SubprocessProtocol, cmd)', 'avoid-shell-true'),
            ('loop.subprocess_shell(asyncio.SubprocessProtocol, cmd, **kwds)', 'avoid-shell-true'),
            ('os.popen("cat")', 'avoid-os-popen'),
            ('os.popen("cat", "r")', 'avoid-os-popen'),
            ('os.popen("cat", "r", 1)', 'avoid-os-popen'),
            ('os.popen("cat", buffering=1)', 'avoid-os-popen'),
            ('os.popen("cat", mode="w")', 'avoid-os-popen'),
            ('os.popen("cat", mode="w", buffering=1)', 'avoid-os-popen'),
        ],
    )
    def test_shell_true_call(self, s, msg_id):
        node = astroid.extract_node(s + ' #@')
        with self.assertAddsMessages(MessageTest(msg_id=msg_id, node=node), ignore_position=True):
            self.checker.visit_call(node)
